@@ -433,7 +433,7 @@ createAppHandlers <- function(httpHandlers, serverFuncSource) {
       })
 
       ws$onClose(function() {
-        shinysession$close()
+        shinysession$wsClosed()
         appsByToken$remove(shinysession$token)
       })
 
@@ -535,8 +535,8 @@ serviceApp <- function() {
 
 #' Run Shiny Application
 #'
-#' Runs a Shiny application. This function normally does not return; interrupt
-#' R to stop the application (usually by pressing Ctrl+C or Esc).
+#' Runs a Shiny application. This function normally does not return; interrupt R
+#' to stop the application (usually by pressing Ctrl+C or Esc).
 #'
 #' The host parameter was introduced in Shiny 0.9.0. Its default value of
 #' \code{"127.0.0.1"} means that, contrary to previous versions of Shiny, only
@@ -547,6 +547,9 @@ serviceApp <- function() {
 #' @param appDir The directory of the application. Should contain
 #'   \code{server.R}, plus, either \code{ui.R} or a \code{www} directory that
 #'   contains the file \code{index.html}. Defaults to the working directory.
+#'   Instead of a directory, this could be a list with \code{ui} and
+#'   \code{server} components, or a Shiny app object created by
+#'   \code{\link{shinyApp}}.
 #' @param port The TCP port that the application should listen on. Defaults to
 #'   choosing a random port.
 #' @param launch.browser If true, the system's default web browser will be
@@ -563,8 +566,8 @@ serviceApp <- function() {
 #'   the value \code{"showcase"}, shows application code and metadata from a
 #'   \code{DESCRIPTION} file in the application directory alongside the
 #'   application. If set to \code{"normal"}, displays the application normally.
-#'   Defaults to \code{"auto"}, which displays the application in the mode
-#'   given in its \code{DESCRIPTION} file, if any.
+#'   Defaults to \code{"auto"}, which displays the application in the mode given
+#'   in its \code{DESCRIPTION} file, if any.
 #'
 #' @examples
 #' \dontrun{
@@ -586,6 +589,20 @@ serviceApp <- function() {
 #'     output$plot <- renderPlot({ hist(runif(input$n)) })
 #'   }
 #' ))
+#'
+#'
+#' # Running a Shiny app object
+#' app <- shinyApp(
+#'   ui = bootstrapPage(
+#'     numericInput('n', 'Number of obs', 100),
+#'     plotOutput('plot')
+#'   ),
+#'   server = function(input, output) {
+#'     output$plot <- renderPlot({ hist(runif(input$n)) })
+#'   }
+#' )
+#'
+#' runApp(app)
 #' }
 #' @export
 runApp <- function(appDir=getwd(),
